'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var core = require('@vue-layout/core');
var vue = require('vue');

function buildPaginationOptions(options) {
    const { buildOrFail , build  } = core.createComponentOptionBuilder('pagination');
    return {
        ...options,
        busy: core.unrefWithDefault(core.extractValueFromOptionValueInput(options.busy), false),
        tag: buildOrFail({
            key: 'tag',
            value: vue.unref(options.tag),
            alt: 'ul'
        }),
        class: buildOrFail({
            key: 'class',
            value: vue.unref(options.class),
            alt: 'pagination'
        }),
        itemTag: buildOrFail({
            key: 'itemTag',
            value: vue.unref(options.itemTag),
            alt: 'li'
        }),
        itemClass: buildOrFail({
            key: 'itemClass',
            alt: 'page-item'
        }),
        linkClass: buildOrFail({
            key: 'linkClass',
            alt: 'page-link'
        }),
        linkActiveClass: buildOrFail({
            key: 'linkActiveClass',
            alt: 'active'
        }),
        prevType: buildOrFail({
            key: 'prevType',
            alt: 'i'
        }),
        prevClass: build({
            key: 'prevClass',
            alt: []
        }),
        prevContent: build({
            key: 'prevContent'
        }),
        nextType: buildOrFail({
            key: 'nextType',
            alt: 'i'
        }),
        nextClass: build({
            key: 'nextClass',
            alt: []
        }),
        nextContent: build({
            key: 'nextContent'
        })
    };
}
function buildPagination(input) {
    const options = buildPaginationOptions(input);
    let totalPages = 1;
    let currentPage = 1;
    if (options.total > 0 && options.limit > 0) {
        totalPages = Math.max(Math.ceil(options.total / options.limit), 1);
        currentPage = Math.floor(options.offset / options.limit) + 1;
    }
    const betweenPages = [];
    for(let i = currentPage - 2; i < currentPage + 2; i++){
        if (i > 0 && i <= totalPages) {
            betweenPages.push(i);
        }
    }
    const loadPage = (page)=>{
        if (options.busy || page === currentPage) return;
        const data = {
            page,
            offset: (page - 1) * options.limit,
            limit: options.limit,
            total: totalPages
        };
        options.load(data);
    };
    const renderPrevPage = ()=>{
        let content;
        if (options.prevClass || options.prevContent) {
            content = vue.h(options.prevType, {
                class: options.prevClass
            }, options.prevContent);
        } else {
            content = [
                currentPage - 1
            ];
        }
        let prevPage = [];
        if (currentPage > 1) {
            prevPage = [
                vue.h(options.itemTag, {
                    class: options.itemClass
                }, [
                    vue.h('button', {
                        class: options.linkClass,
                        disabled: options.busy,
                        onClick ($event) {
                            $event.preventDefault();
                            return loadPage(currentPage - 1);
                        }
                    }, [
                        content
                    ])
                ])
            ];
        }
        return prevPage;
    };
    const renderBetweenPages = ()=>{
        const children = [];
        for(let i = 0; i < betweenPages.length; i++){
            const node = vue.h(options.itemTag, {
                class: options.itemClass
            }, [
                vue.h('button', {
                    ...vue.mergeProps({
                        ...betweenPages[i] === currentPage ? {
                            class: options.linkActiveClass
                        } : {}
                    }, {
                        class: options.linkClass
                    }),
                    disabled: options.busy,
                    onClick ($event) {
                        $event.preventDefault();
                        // eslint-disable-next-line prefer-rest-params
                        return loadPage(betweenPages[i]);
                    }
                }, [
                    betweenPages[i]
                ])
            ]);
            children.push(node);
        }
        return children;
    };
    const renderNextPage = ()=>{
        let nextPage = [];
        let content;
        if (options.nextClass || options.nextContent) {
            content = vue.h(options.nextType, {
                class: options.nextClass
            }, options.nextContent);
        } else {
            content = [
                currentPage + 1
            ];
        }
        if (currentPage < totalPages) {
            nextPage = [
                vue.h(options.itemTag, {
                    class: options.itemClass
                }, [
                    vue.h('button', {
                        class: options.linkClass,
                        disabled: options.busy,
                        onClick ($event) {
                            $event.preventDefault();
                            return loadPage(currentPage + 1);
                        }
                    }, [
                        content
                    ])
                ])
            ];
        }
        return nextPage;
    };
    return vue.h(options.tag, {
        class: options.class
    }, [
        renderPrevPage(),
        renderBetweenPages(),
        renderNextPage()
    ]);
}

const Pagination = vue.defineComponent({
    name: 'Pagination',
    props: {
        total: {
            type: Number,
            default: 0
        },
        limit: {
            type: Number,
            default: 0
        },
        offset: {
            type: Number,
            default: 0
        },
        busy: {
            type: Boolean,
            default: false
        },
        meta: {
            type: Object,
            default: undefined
        }
    },
    emits: [
        'load'
    ],
    setup (props, { emit  }) {
        const meta = vue.computed(()=>{
            if (typeof props.meta === 'undefined') {
                return {};
            }
            return props.meta;
        });
        return ()=>buildPagination({
                total: meta.value.total ?? props.total,
                limit: meta.value.limit ?? props.limit,
                offset: meta.value.offset ?? props.offset,
                busy: meta.value.busy ?? props.busy,
                load: (data)=>emit('load', data)
            });
    }
});

function install(instance, options) {
    options ?? (options = {});
    core.applyPluginBaseOptions(options);
    instance.component('Pagination', Pagination);
}
var index = {
    install
};

exports.Pagination = Pagination;
exports.buildPagination = buildPagination;
exports.buildPaginationOptions = buildPaginationOptions;
exports.default = index;
exports.install = install;
module.exports = Object.assign(exports.default, exports);
//# sourceMappingURL=index.cjs.map
